'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var lr = require('@lezer/lr');
var highlight = require('@lezer/highlight');

// This file was generated by lezer-generator. You probably shouldn't edit it.
const Content = 1,
  StartTag = 2,
  StartAmpersandTag = 3,
  StartPartialTag = 4,
  StartSectionTag = 5,
  StartCloseSectionTag = 6,
  MismatchedStartCloseSectionTag = 7,
  StartCommentTag = 8,
  StartUnescapedTag = 9,
  DelimiterTag = 10,
  InvalidDelimiterTag = 11,
  TagKey = 12,
  InvalidTagKey = 13,
  Comment = 14,
  EndTag = 15,
  EndUnescapedTag = 16,
  SectionTag = 24,
  CloseSectionTag = 25;

/* Hand-written tokenizer for mustache tag matching. */

function tagKeyChar(ch) {
  return (
    ch == 45 ||
    ch == 46 ||
    (ch >= 48 && ch <= 57) ||
    (ch >= 65 && ch <= 90) ||
    ch == 95 ||
    (ch >= 97 && ch <= 122)
  );
}

function delimiterChar(ch) {
  return ch != 32 && ch != 61 && ch > 0;
}

let cachedKey = null,
  cachedInput = null,
  cachedPos = 0;
function tagKeyAfter(input, offset, token) {
  let pos = input.pos + offset;
  if (cachedInput == input && cachedPos == pos) return cachedKey;
  while (input.peek(offset) == 32 /* ' ' */) offset++;
  let first = token.charCodeAt(0);
  let key = "";
  for (;;) {
    let next = input.peek(offset);
    if (!tagKeyChar(next)) break;
    if (next == first) {
      let len = 1;
      for (let l = token.length; len < l; len++)
        if (input.peek(offset + len) != token.charCodeAt(len)) break;
      if (len == token.length) break;
    }
    key += String.fromCharCode(next);
    offset++;
  }
  cachedInput = input;
  cachedPos = pos;
  return (cachedKey = key || null);
}

let cachedDelimiters = null,
  cachedDelimiterInput = null,
  cachedDelimiterPos = 0;
function delimitersAfter(input, offset) {
  let pos = input.pos + offset;
  if (cachedDelimiterInput == input && cachedDelimiterPos == pos)
    return cachedDelimiters;
  let delimiter = "";
  let startDelimiter = "";
  let endDelimiter = "";
  for (;;) {
    let next = input.peek(offset);
    if (!delimiterChar(next)) {
      if (next == 32 /* ' ' */ && startDelimiter == "") {
        startDelimiter = delimiter;
        delimiter = "";
        offset++;
        continue;
      } else if (next == 61 /* '=' */ && startDelimiter != "") {
        endDelimiter = delimiter;
        break;
      } else if (next < 0) {
        return;
      }
    }
    delimiter += String.fromCharCode(next);
    offset++;
  }
  if (startDelimiter == "" || endDelimiter == "") return null;
  cachedDelimiterInput = input;
  cachedDelimiterPos = pos;
  return (cachedDelimiters = [startDelimiter, endDelimiter] || null);
}

function Section(key, parent) {
  this.key = key;
  this.parent = parent;
}

function MustacheContext(section, delimiters) {
  this.section = section;
  this.delimiters = delimiters;
}

const mustacheContext = new lr.ContextTracker({
  start: new MustacheContext(null, ["{{", "}}"]),
  shift(context, term, stack, input) {
    return term == StartSectionTag
      ? new MustacheContext(
          new Section(
            tagKeyAfter(input, 3, context.delimiters[1]) || "",
            context.section,
          ),
          context.delimiters,
        )
      : term == DelimiterTag
        ? new MustacheContext(
            context.section,
            delimitersAfter(input, context.delimiters[1].length + 1) ??
              context.delimiters,
          )
        : context;
  },
  reduce(context, term) {
    return term == CloseSectionTag && context.section
      ? new MustacheContext(context.section?.parent, context.delimiters)
      : context;
  },
  reuse(context, node, _stack, input) {
    let type = node.type.id;
    return type == StartSectionTag || type == SectionTag
      ? new MustacheContext(
          new Section(
            tagKeyAfter(input, 3, context.delimiters[1]) || "",
            context.section,
          ),
          context.delimiters,
        )
      : context;
  },
  strict: false,
});

const content = new lr.ExternalTokenizer(
  (input, stack) => {
    if (scanToToken(input, stack.context.delimiters[0])) {
      input.acceptToken(Content);
    }
  },
  { contextual: true },
);

const startTag = new lr.ExternalTokenizer(
  (input, stack) => {
    let startDelimiter = stack.context.delimiters[0];
    if (scanToken(input, startDelimiter)) {
      let next = input.next;
      if (next == 47 /* '/' */) {
        input.advance();
        let key = tagKeyAfter(input, 0, startDelimiter);
        if (!key) return input.acceptToken(StartCloseSectionTag);
        if (stack.context && key == stack.context.section?.key)
          return input.acceptToken(StartCloseSectionTag);
        for (let s = stack.context.section; s; s = s.parent)
          if (s.key == key)
            return input.acceptToken(MismatchedStartCloseSectionTag);
        input.acceptToken(MismatchedStartCloseSectionTag);
      } else if (next == 35 /* '#' */ || next == 94 /* '^' */) {
        input.advance();
        input.acceptToken(StartSectionTag);
      } else if (next == 33 /* '!' */) {
        input.advance();
        input.acceptToken(StartCommentTag);
      } else if (next == 123 /* '{' */) {
        input.advance();
        input.acceptToken(StartUnescapedTag);
      } else if (next == 61 /* '=' */) {
        input.advance();
        input.acceptToken(
          scanDelimiterTag(input, stack.context.delimiters[1])
            ? DelimiterTag
            : InvalidDelimiterTag,
        );
      } else if (next == 38 /* '&' */) {
        input.advance();
        input.acceptToken(StartAmpersandTag);
      } else if (next == 62 /* '>' */) {
        input.advance();
        input.acceptToken(StartPartialTag);
      } else input.acceptToken(StartTag);
    }
  },
  { contextual: true },
);

const tagKey = new lr.ExternalTokenizer(
  (input, stack) => {
    if (input.next == 32 /* ' ' */) return;
    let key = tagKeyAfter(input, 0, stack.context.delimiters[1]);
    let len = key?.length ?? 0;
    for (let i = 0; i < len; i++) input.advance();
    if (len > 1 && (key.startsWith(".") || key.indexOf("..") >= 0))
      input.acceptToken(InvalidTagKey);
    else if (len) input.acceptToken(TagKey);
  },
  { contextual: true },
);

const comment = new lr.ExternalTokenizer(
  (input, stack) => {
    if (scanToToken(input, stack.context.delimiters[1])) {
      input.acceptToken(Comment);
    }
  },
  { contextual: true },
);

const endTag = new lr.ExternalTokenizer(
  (input, stack) => {
    if (scanToken(input, stack.context.delimiters[1]))
      input.acceptToken(EndTag);
  },
  { contextual: true },
);
const endUnescapedTag = new lr.ExternalTokenizer(
  (input, stack) => {
    if (scanToken(input, "}" + stack.context.delimiters[1]))
      input.acceptToken(EndUnescapedTag);
  },
  { contextual: true },
);

function scanToken(input, token) {
  for (let i = 0, l = token.length; i < l; i++) {
    if (input.peek(i) != token.charCodeAt(i)) return false;
  }
  for (let i = 0, l = token.length; i < l; i++) input.advance();
  return true;
}

function scanToToken(input, token) {
  let len = 0,
    first = token.charCodeAt(0);
  scan: for (; ; input.advance(), len++) {
    if (input.next < 0) break;
    if (input.next == first) {
      for (let i = 1; i < token.length; i++)
        if (input.peek(i) != token.charCodeAt(i)) continue scan;
      break;
    }
  }
  return len > 0;
}

function scanDelimiterTag(input, endToken) {
  if (input.next == 32 /* ' ' */) return false;
  while (delimiterChar(input.next)) input.advance();
  if (input.next == 32 /* ' ' */) input.advance();
  else return false;
  while (delimiterChar(input.next)) input.advance();
  if (input.next == 61 /* '=' */) input.advance();
  else return false;
  return scanToken(input, endToken);
}

const mustacheHighlighting = highlight.styleTags({
  "StartTag StartSectionTag StartCloseSectionTag MismatchedStartCloseSectionTag StartCommentTag StartPartialTag StartUnescapedTag StartAmpersandTag EndTag EndUnescapedTag DelimiterTag InvalidDelimiterTag":
    highlight.tags.strong,
  Content: highlight.tags.content,
  TagKey: highlight.tags.tagName,
  InvalidTagKey: [highlight.tags.tagName, highlight.tags.invalid],
  Comment: highlight.tags.blockComment,
});

// This file was generated by lezer-generator. You probably shouldn't edit it.
const parser = lr.LRParser.deserialize({
  version: 14,
  states: "+|QQOROOOrO!fO'#CzOOOR'#Cn'#CnO}O!fO'#C|OOOR'#Co'#CoO!YOhO'#CpO!bO!fO'#C}OOOR'#Cq'#CqO!mO!fO'#CqO!xO!fO'#DOOOOR'#Cr'#CrO#TO!fO'#DPOOOR'#Ct'#CtO#`ORO'#CsOOOR'#Cy'#CyOOOR'#Cv'#CvQQOROOOOO!U'#Cw'#CwO#gO!fO,59fO#rO!rO,59fO#zO!fO,59hO$VO!rO,59hOOOR,59[,59[O$_O`O,59[O$dO!fO,59iO$oO!rO,59iO$wO!fO,59]O%SO#SO,59]O%[O!fO,59jO%gO!rO,59jO%oO!fO,59kO%zO!rO,59kO&SO!fO'#DQOOOR'#Cu'#CuOOOR,59_,59_O#`ORO,59_OOOR-E6t-E6tOOO!U-E6u-E6uO&_O!rO1G/QO&_O!rO1G/QOOOR1G/Q1G/QO&gO!rO1G/SO&gO!rO1G/SOOOR1G/S1G/SOOOR1G.v1G.vO&oO!rO1G/TO&oO!rO1G/TOOOR1G/T1G/TO&wO#SO1G.wO&wO#SO1G.wOOOR1G.w1G.wO'PO!rO1G/UO'PO!rO1G/UOOOR1G/U1G/UO'XO!rO1G/VO'XO!rO1G/VOOOR1G/V1G/VO'aO!fO,59lO'lO!rO,59lOOOR1G.y1G.yO'tO!rO7+$lOOOR7+$l7+$lO'|O!rO7+$nOOOR7+$n7+$nO(UO!rO7+$oOOOR7+$o7+$oO(^O#SO7+$cOOOR7+$c7+$cO(fO!rO7+$pOOOR7+$p7+$pO(nO!rO7+$qOOOR7+$q7+$qO(vO!rO1G/WO(vO!rO1G/WOOOR1G/W1G/WOOOR<<HW<<HWOOOR<<HY<<HYOOOR<<HZ<<HZOOOR<<G}<<G}OOOR<<H[<<H[OOOR<<H]<<H]O)OO!rO7+$rOOOR7+$r7+$rOOOR<<H^<<H^",
  stateData: ")W~OP^OQPORUOSROTZOVXOWTOXWOY^OZ^O~O[cO]cOoaO~O[eO]eOoaO~O^gO_fO~O[iO]iOoaO~O[kO]kOoaO~O[mO]mOoaO~O[oO]oOoaO~OUpO~PQO[vO]vOoaO~O_xOoaO~O[yO]yOoaO~O_{OoaO~O_|O~O[}O]}OoaO~O_!POoaO~O[!QO]!QOoaO~O`!SOoaO~O[!TO]!TOoaO~O_!VOoaO~O[!WO]!WOoaO~O_!YOoaO~O[![O]![OoaO~O_!_OoaO~O_!aOoaO~O_!cOoaO~O`!eOoaO~O_!gOoaO~O_!iOoaO~O[!jO]!jOoaO~O_!lOoaO~O_!mOoaO~O_!nOoaO~O_!oOoaO~O`!pOoaO~O_!qOoaO~O_!rOoaO~O_!tOoaO~O_!uOoaO~O",
  goto: "%WuPPPPPPPPPPPPPPPPPPvvvvvv|!S!Y!dP$^$dP$j$p$v$|%SX^O]`sX]O]`sQr]R!]sQ`OQs]Tt`sQbPQdRQhUQjWQlXQnZzubdhjlnwz!O!R!U!X!Z!^!`!b!d!f!h!k!sQwcQzeQ!OiQ!RkQ!UmQ!XoQ!ZpQ!^vQ!`yQ!b}Q!d!QQ!f!TQ!h!WQ!k![R!s!jX_O]`sXQO]`sXSO]`sXVO]`sXYO]`sX[O]`sTq]s",
  nodeNames: "⚠ Content StartTag StartAmpersandTag StartPartialTag StartSectionTag StartCloseSectionTag MismatchedStartCloseSectionTag StartCommentTag StartUnescapedTag DelimiterTag InvalidDelimiterTag TagKey InvalidTagKey Comment EndTag EndUnescapedTag Document Tag PartialTag CommentTag UnescapedTag MismatchedCloseSectionTag Element SectionTag CloseSectionTag",
  maxTerm: 36,
  context: mustacheContext,
  nodeProps: [
    ["closedBy", -7,2,3,4,5,6,7,8,"EndTag",9,"EndUnescapedTag",24,"CloseSectionTag"],
    ["openedBy", 15,"StartTag StartSectionTag StartCloseSectionTag MismatchedStartCloseSectionTag StartCommentTag",16,"StartUnescapedTag",25,"SectionTag"]
  ],
  propSources: [mustacheHighlighting],
  skippedNodes: [0],
  repeatNodeCount: 2,
  tokenData: "Z~RPpqU~ZOo~",
  tokenizers: [content, startTag, tagKey, comment, endTag, endUnescapedTag, 0],
  topRules: {"Document":[0,17]},
  tokenPrec: 0
});

exports.parser = parser;
